@@ -33,17 +33,19 @@
 #define PLUGIN_NAME "xfce4-notes-plugin"
 
 
-static void     note_page_load_data (XfcePanelPlugin *, NotePage *);
-static gboolean on_note_delete ();
-static void     on_note_close (GtkWidget *, GtkToggleButton *);
-static gboolean on_title_press (GtkWidget *, GdkEventButton *, GtkWindow *);
-static gboolean on_title_scroll (GtkWidget *, GdkEventScroll *, Note *);
-static gboolean on_note_key_press (GtkWidget *, GdkEventKey *, NotesPlugin *);
-static void     on_note_changed (GtkWidget *, NotesPlugin *);
-static void     on_page_create (GtkWidget *, NotesPlugin *);
-static gboolean on_page_delete (GtkWidget *, NotesPlugin *);
-static void     note_page_destroy (GtkWidget *, gint response_id,
-                                   NotesPlugin *);
+static void     note_page_load_data     (XfcePanelPlugin *, NotePage *);
+static void     on_notes_close             (GtkWidget *, NotesPlugin *);
+static gboolean on_note_window_close     ();
+static gboolean on_title_press             (GtkWidget *, GdkEventButton *, GtkWindow *);
+static gboolean on_title_scroll         (GtkWidget *, GdkEventScroll *, Note *);
+static gboolean on_note_key_press         (GtkWidget *, GdkEventKey *, NotesPlugin *);
+static void     on_note_changed         (GtkWidget *, NotesPlugin *);
+static void     note_page_rename         (Note *);
+static gboolean on_note_rename             (GtkWidget *, GdkEventButton *, Note *);
+static void     on_note_rename_response (GtkDialog *, gint response, GSList *);
+static void     on_page_create             (GtkWidget *, NotesPlugin *);
+static gboolean on_page_delete             (GtkWidget *, NotesPlugin *);
+static void     note_page_destroy         (GtkWidget *, gint response_id, NotesPlugin *);
 
 Note *
 note_new (NotesPlugin *notes)
@@ -56,6 +58,9 @@ note_new (NotesPlugin *notes)
     note = g_new0 (Note, 1);
     note->pages = NULL;
 
+    GtkAccelGroup *accel_group;
+    accel_group = gtk_accel_group_new ();
+
     /* Window */
     note->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
@@ -63,10 +68,13 @@ note_new (NotesPlugin *notes)
     gtk_window_set_default_size (GTK_WINDOW (note->window), 242, 200);
     gtk_window_set_decorated (GTK_WINDOW (note->window), FALSE);
     gtk_window_set_icon_name (GTK_WINDOW (note->window), GTK_STOCK_EDIT);
+    gtk_window_add_accel_group (GTK_WINDOW (note->window), accel_group);
+
+    gtk_widget_set_name (note->window, "xfce4-notes-plugin");
 
     /* Prevent close window */
-    g_signal_connect (note->window, "delete-event", G_CALLBACK (on_note_delete),
-                      NULL);
+    g_signal_connect (note->window, "delete-event",
+                      G_CALLBACK (on_note_window_close), NULL);
 
 
     /* Frame */
@@ -96,7 +104,7 @@ note_new (NotesPlugin *notes)
     gtk_widget_show (note->add);
 
     gtk_tooltips_set_tip (GTK_TOOLTIPS (notes->tooltips), note->add,
-                          _("Open a new page"), NULL);
+                          _("Create new note"), NULL);
     gtk_widget_set_size_request (note->add, 22, 22);
     gtk_box_pack_start (GTK_BOX (note->hbox), note->add, FALSE, FALSE, 0);
 
@@ -105,6 +113,8 @@ note_new (NotesPlugin *notes)
 
     gtk_container_add (GTK_CONTAINER (note->add), image_add);
 
+    gtk_widget_add_accelerator (note->add, "clicked", accel_group, 'N',
+                                GDK_CONTROL_MASK, GTK_ACCEL_MASK);
     g_signal_connect (note->add, "clicked", G_CALLBACK (on_page_create), notes);
 
     /* Remove button */
@@ -112,7 +122,7 @@ note_new (NotesPlugin *notes)
     gtk_widget_show (note->del);
 
     gtk_tooltips_set_tip (GTK_TOOLTIPS (notes->tooltips), note->del,
-                          _("Delete a page"), NULL);
+                          _("Delete this note"), NULL);
     gtk_widget_set_size_request (note->del, 22, 22);
     gtk_box_pack_start (GTK_BOX (note->hbox), note->del, FALSE, FALSE, 0);
 
@@ -121,10 +131,14 @@ note_new (NotesPlugin *notes)
 
     gtk_container_add (GTK_CONTAINER (note->del), image_del);
 
+    gtk_widget_add_accelerator (note->del, "clicked", accel_group, 'W',
+                                GDK_CONTROL_MASK, GTK_ACCEL_MASK);
     g_signal_connect (note->del, "clicked", G_CALLBACK (on_page_delete), notes);
 
     /* Event box move + Title */
     note->move_event_box = gtk_event_box_new ();
+    gtk_event_box_set_visible_window (GTK_EVENT_BOX (note->move_event_box),
+                                      FALSE);
     gtk_widget_show (note->move_event_box);
 
     gtk_box_pack_start (GTK_BOX (note->hbox), note->move_event_box, TRUE, TRUE,
@@ -163,8 +177,8 @@ note_new (NotesPlugin *notes)
 
     gtk_container_add (GTK_CONTAINER (note->close), image_close);
 
-    g_signal_connect (note->close, "clicked", G_CALLBACK (on_note_close),
-                      notes->button);
+    g_signal_connect (note->close, "clicked",
+                      G_CALLBACK (on_notes_close), notes);
 
 
     /* Notebook */
@@ -183,28 +197,31 @@ note_new (NotesPlugin *notes)
 void
 note_page_new (XfcePanelPlugin *plugin, NotesPlugin *notes)
 {
-	NotePage *page;
-	Note *note;
+    NotePage *page;
+    Note *note;
     GtkTextBuffer *buffer;
-    gchar note_id[8];
+    gint id;
+    gchar label[8];
 
     DBG ("Create a new page");
 
-	page = g_new0 (NotePage, 1);
-	note = notes->note;
+    page = g_new0 (NotePage, 1);
+    note = notes->note;
     note->pages = g_list_append (note->pages, page);
-
-    /* HBox */
-    page->hbox = gtk_hbox_new (FALSE, 0);
-    gtk_widget_show (page->hbox);
+    id = g_list_length (note->pages);
 
     /* Label */
-    g_snprintf (note_id, 8, "%d", g_list_length (note->pages));
-    page->label = gtk_label_new (note_id);
+    GtkWidget *eventbox;
+    eventbox = gtk_event_box_new ();
+    gtk_widget_show (eventbox);
+
+    g_snprintf (label, 8, "%d", id);
+    page->label = gtk_label_new (label);
     gtk_widget_show (page->label);
 
-    gtk_label_set_justify (GTK_LABEL (page->label), GTK_JUSTIFY_RIGHT);
-    gtk_box_pack_start (GTK_BOX (page->hbox), page->label, TRUE, TRUE, 0);
+    gtk_event_box_set_visible_window (GTK_EVENT_BOX (eventbox), FALSE);
+    gtk_container_add (GTK_CONTAINER (eventbox), page->label);
+    gtk_container_set_border_width (GTK_CONTAINER (eventbox), 3);
 
     /* Scrolled window + Text view */
     page->scroll = gtk_scrolled_window_new (NULL, NULL);
@@ -223,24 +240,24 @@ note_page_new (XfcePanelPlugin *plugin, NotesPlugin *notes)
 
     /* Append the widget to the notebook */
     page->id = gtk_notebook_append_page (GTK_NOTEBOOK (note->notebook),
-                                         page->scroll, page->hbox);
+                                         page->scroll, eventbox);
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (note->notebook),
                                 (gboolean) page->id);
 
-
     note_page_load_data (plugin, page);
 
     g_signal_connect (page->text, "key-press-event",
                       G_CALLBACK (on_note_key_press), notes);
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->text));
     g_signal_connect (buffer, "changed", G_CALLBACK (on_note_changed), notes);
+    g_signal_connect (eventbox, "button-press-event",
+                      G_CALLBACK (on_note_rename), note);
 }
 
 static void
 note_page_load_data (XfcePanelPlugin *plugin, NotePage *page)
 {
     gchar *file;
-    gchar note_entry[12];
     XfceRc *rc;
 
     if (!(file = xfce_panel_plugin_lookup_rc_file (plugin)))
@@ -253,8 +270,19 @@ note_page_load_data (XfcePanelPlugin *plugin, NotePage *page)
 
     if (rc)
       {
+        gchar note_entry[12], label_entry[13];
         GtkTextBuffer *buffer;
-        const gchar *text;
+        const gchar *text, *label;
+
+        page->label_dirty = FALSE;
+        g_snprintf (label_entry, 13, "label%d", page->id);
+        if (xfce_rc_has_entry (rc, label_entry))
+          {
+            page->label_dirty = TRUE;
+            label = xfce_rc_read_entry (rc, label_entry, ":)");
+            gtk_label_set_text (GTK_LABEL (page->label), label);
+            gtk_label_set_angle (GTK_LABEL (page->label), 90);
+          }
 
         g_snprintf (note_entry, 12, "note%d", page->id);
         text = xfce_rc_read_entry (rc, note_entry, "");
@@ -263,26 +291,25 @@ note_page_load_data (XfcePanelPlugin *plugin, NotePage *page)
 
         buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->text));
         gtk_text_buffer_set_text (buffer, text, -1);
-
         gtk_text_view_set_buffer (GTK_TEXT_VIEW (page->text), buffer);
 
         xfce_rc_close (rc);
       }
 }
 
+static void
+on_notes_close (GtkWidget *widget, NotesPlugin* notes)
+{
+    gtk_button_clicked (GTK_BUTTON (notes->button));
+}
+
 static gboolean
-on_note_delete ()
+on_note_window_close ()
 {
     /* Prevent close window (Alt-F4) */
     return TRUE;
 }
 
-static void
-on_note_close (GtkWidget *widget, GtkToggleButton *panel_button)
-{
-    gtk_toggle_button_set_active (panel_button, FALSE);
-}
-
 static gboolean
 on_title_press (GtkWidget *widget, GdkEventButton *event, GtkWindow *window)
 {
@@ -324,8 +351,22 @@ on_title_scroll (GtkWidget *widget, GdkEventScroll *event, Note *note)
 static gboolean
 on_note_key_press (GtkWidget *widget, GdkEventKey *event, NotesPlugin *notes)
 {
-    if (event->type == GDK_KEY_PRESS && event->keyval == GDK_Escape)
-        on_note_close (widget, GTK_TOGGLE_BUTTON (notes->button));
+    if (event->type == GDK_KEY_PRESS)
+      {
+        if (event->keyval == GDK_Escape)
+            gtk_button_clicked (GTK_BUTTON (notes->button));
+        else if (event->state & GDK_CONTROL_MASK)
+          {
+            if (event->keyval == GDK_Page_Down)
+                gtk_notebook_next_page (GTK_NOTEBOOK (notes->note->notebook));
+            else if (event->keyval == GDK_Page_Up)
+                gtk_notebook_prev_page (GTK_NOTEBOOK (notes->note->notebook));
+          }
+        else if (event->keyval == GDK_F2)
+          {
+            note_page_rename (notes->note);
+          }
+      }
 
     return FALSE;
 }
@@ -333,16 +374,104 @@ on_note_key_press (GtkWidget *widget, GdkEventKey *event, NotesPlugin *notes)
 static void
 on_note_changed (GtkWidget *widget, NotesPlugin *notes)
 {
-    if (notes->timeout_id > 0)
+    save_on_timeout (notes);
+}
+
+
+
+/*
+ * Rename a tab-label
+ */
+
+static void
+note_page_rename (Note *note)
+{
+    DBG ("Rename the note_page_menu_label");
+
+    GSList *slist = NULL;
+    gint id;
+    GtkWidget *dialog, *vbox;
+    GtkWidget *label, *entry;
+    NotePage *page;
+
+    id = gtk_notebook_get_current_page (GTK_NOTEBOOK (note->notebook));
+    page = (NotePage *)g_list_nth_data (note->pages, id);
+    label = page->label;
+
+    dialog =
+        gtk_dialog_new_with_buttons (_("Rename"),
+                                     GTK_WINDOW (note->window),
+                                     GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
+                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                     GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+    gtk_window_set_icon_name (GTK_WINDOW (dialog), GTK_STOCK_EDIT);
+
+    vbox = gtk_vbox_new (2, FALSE);
+    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
+    gtk_widget_show (vbox);
+
+    gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+
+    entry = gtk_entry_new ();
+    gtk_entry_set_text (GTK_ENTRY (entry),
+                        gtk_label_get_text (GTK_LABEL (label)));
+    gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+    gtk_widget_show (entry);
+
+    gtk_container_add (GTK_CONTAINER (vbox), entry);
+
+    slist = g_slist_append (slist, entry);
+    slist = g_slist_append (slist, page);
+
+    g_signal_connect (dialog, "response",
+                      G_CALLBACK (on_note_rename_response), slist);
+
+    gtk_widget_show (dialog);
+}
+
+static gboolean
+on_note_rename (GtkWidget *widget, GdkEventButton *event, Note *note)
+{
+    if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
       {
-        g_source_remove (notes->timeout_id);
-        notes->timeout_id = 0;
+        note_page_rename (note);
       }
 
-    notes->timeout_id = g_timeout_add (60000, (GSourceFunc) save_on_timeout,
-                                       notes);
+    return FALSE;
 }
 
+static void
+on_note_rename_response (GtkDialog *dialog, gint response, GSList *slist)
+{
+    GtkWidget *entry;
+    NotePage *page;
+
+    if (response == GTK_RESPONSE_OK)
+      {
+        entry = GTK_WIDGET (g_slist_nth_data (slist, 0));
+        page = (NotePage *) g_slist_nth_data (slist, 1);
+
+        DBG ("Rename to: %s", gtk_entry_get_text (GTK_ENTRY (entry)));
+
+        page->label_dirty = TRUE;
+        gtk_label_set_text (GTK_LABEL (page->label),
+                            gtk_entry_get_text (GTK_ENTRY (entry)));
+        gtk_label_set_angle (GTK_LABEL (page->label), 90);
+      }
+
+    g_slist_free (slist);
+    gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+
+
+/*
+ * Create / Destroy a page
+ */
+
 static void
 on_page_create (GtkWidget *widget, NotesPlugin *notes)
 {
@@ -357,9 +486,6 @@ on_page_delete (GtkWidget *widget, NotesPlugin *notes)
     NotePage *page;
     GtkTextBuffer *buffer;
 
-    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (notes->note->notebook)) == 1)
-        return TRUE;
-
     id = gtk_notebook_get_current_page (GTK_NOTEBOOK (notes->note->notebook));
     page = (NotePage *)g_list_nth_data (notes->note->pages, id);
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->text));
@@ -387,22 +513,50 @@ on_page_delete (GtkWidget *widget, NotesPlugin *notes)
 static void
 note_page_destroy (GtkWidget *widget, gint response_id, NotesPlugin *notes)
 {
-    if (widget != NULL);
+    gchar *file;
+    XfceRc *rc;
+    gint id;
+    GtkNotebook *notebook;
+    GList *pages;
+    NotePage *page;
+    GtkTextBuffer *buffer;
+    gchar tab_label[8];
+    gchar note_entry[12], label_entry[13];
+
+    if (GTK_IS_WIDGET (widget))
         gtk_widget_destroy (widget);
 
     if (response_id == GTK_RESPONSE_YES)
       {
-      	gint id;
-        GtkNotebook *notebook;
-        GList *pages;
-        NotePage *page;
-        gchar tab_label[8];
-
         notebook = GTK_NOTEBOOK (notes->note->notebook);
         id = gtk_notebook_get_current_page (notebook);
 
         DBG ("Delete id %d", id);
 
+        if (!(file = xfce_panel_plugin_save_location (notes->plugin, TRUE)))
+            return;
+
+        rc = xfce_rc_simple_open (file, FALSE);
+        g_free (file);
+
+        if (gtk_notebook_get_n_pages (notebook) == 1)
+          {
+            page = (NotePage *)g_list_nth_data (notes->note->pages, id);
+            buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->text));
+            gtk_text_buffer_set_text (buffer, "", -1);
+
+            gtk_label_set_text (GTK_LABEL (page->label), "1");
+            gtk_label_set_angle (GTK_LABEL (page->label), 0);
+            if (rc)
+              {
+                xfce_rc_delete_entry (rc, "label0", TRUE);
+                xfce_rc_close (rc);
+                page->label_dirty = FALSE;
+              }
+
+            return;
+          }
+
         gtk_notebook_remove_page (notebook, id);
 
         notes->note->pages = g_list_nth (notes->note->pages, 0);
@@ -420,25 +574,20 @@ note_page_destroy (GtkWidget *widget, gint response_id, NotesPlugin *notes)
 
             DBG ("id:%d", id);
 
-            g_snprintf (tab_label, 8, "%d", id+1);
-            gtk_label_set_text (GTK_LABEL (page->label), tab_label);
+            if (!page->label_dirty)
+              {
+                g_snprintf (tab_label, 8, "%d", id+1);
+                gtk_label_set_text (GTK_LABEL (page->label), tab_label);
+              }
           }
 
-        gchar *file;
-        gchar note_entry[12];
-        XfceRc *rc;
-
-        if (!(file = xfce_panel_plugin_save_location (notes->plugin, TRUE)))
-            return;
-
-        rc = xfce_rc_simple_open (file, FALSE);
-        g_free (file);
-
         if (rc)
           {
             g_snprintf (note_entry, 12, "note%d", g_list_length (pages));
+            g_snprintf (label_entry, 13, "label%d", g_list_length (pages));
 
             xfce_rc_delete_entry (rc, note_entry, TRUE);
+            xfce_rc_delete_entry (rc, label_entry, TRUE);
             xfce_rc_close (rc);
           }
 
