@@ -30,7 +30,6 @@ typedef struct
     gboolean task_switcher;
     gboolean always_on_top;
     gboolean stick;
-    gboolean vscrollbar;
 }
 NotesOptions;
 
@@ -52,6 +51,6 @@ NotesPlugin;
 NotesPlugin *   notes_new (XfcePanelPlugin *);
 Note *          note_new (NotesPlugin *);
 void            note_page_new (XfcePanelPlugin *, NotesPlugin *);
-gboolean        save_on_timeout (NotesPlugin *);
+void            save_on_timeout (NotesPlugin *);
 
 #endif
